@@ -9,13 +9,18 @@
 #ifndef __DasherAppInterface_h__
 #define __DasherAppInterface_h__
 
-#include "DasherScreen.h"
-#include "DashEdit.h"
-#include "AlphIO.h"
+#include "Alphabet/AlphIO.h"
+#include "DasherInput.h"
 #include <string>
 #include <vector>
 
-namespace Dasher {class CDasherAppInterface;}
+namespace Dasher
+{
+	class CDasherAppInterface;
+	class CDasherScreen;
+	class CDashEditbox;
+}
+
 class Dasher::CDasherAppInterface
 {
 public:
@@ -46,7 +51,7 @@ public:
 
 	//! Tell the interface how many font sizes there are and what they are
 
-	virtual void GetFontSizes(std::vector<int> *FontSizes)=0;
+	virtual void GetFontSizes(std::vector<int> *FontSizes) const =0;
 
 	//! Provides the number of characters per minute input. (Not implemented)
 	virtual double GetCurCPM()=0;
@@ -76,6 +81,8 @@ public:
     virtual int GetOneButton()=0;
     virtual int GetAutoOffset()=0;
     virtual void SetOneButton(int Value)=0;
+
+	virtual void SetInput( CDasherInput *_pInput ) = 0;
 };
 
 
