@@ -8,12 +8,14 @@
 
 #include "../Common/Common.h"
 
-#include "DasherInterface.h"
-
-#include "CustomAlphabet.h"
 #include "CustomColours.h"
 #include "DasherViewSquare.h"
-#include "PPMLanguageModel.h"
+
+#include <iostream>
+
+#include "DasherInterface.h"
+
+
 namespace {
 	#include "stdio.h"
 }
@@ -24,22 +26,20 @@ const string CDasherInterface::EmptyString = "";
 
 
 CDasherInterface::CDasherInterface()
-	: 
+: 
 m_Alphabet(0), 
-m_Colours(0), 
-m_LanguageModel(0), 
-m_DasherModel(0), 
+m_pColours(0), 
+m_pDasherModel(0), 
 m_DashEditbox(0), 
 m_DasherScreen(0), 
-m_DasherView(0), 
+m_pDasherView(0), 
 m_SettingsStore(0), 
 m_SettingsUI(0),
 m_AlphIO(0), 
 m_ColourIO(0), 
-TrainContext(0), 
 AlphabetID(""), 
-LanguageModelID(-1), 
-ViewID(-1),
+m_LanguageModelID(-1), 
+m_ViewID(-1),
 m_MaxBitRate(-1), 
 m_DrawKeyboard(false), 
 m_ColourMode(0), 
@@ -52,22 +52,23 @@ m_TrainFile(""),
 m_DasherFont(""), 
 m_DasherFontSize(Opts::Normal), 
 m_EditFont(""), 
-m_EditFontSize(0)
+m_EditFontSize(0),
+m_pInput(NULL)
+
 {
+  m_Params = new CLanguageModelParams;
 }
 
 
 CDasherInterface::~CDasherInterface()
 {
-	if (m_LanguageModel)
-		m_LanguageModel->ReleaseContext(TrainContext);
-	delete m_DasherModel;   // The order of some of these deletions matters
-	delete m_LanguageModel; // eg DasherModel has a pointer to LanguageModel.
-	delete m_Alphabet;      // DM baulks if LM is deleted before it is.
-	delete m_DasherView;
+	delete m_pDasherModel;   // The order of some of these deletions matters
+	delete m_Alphabet;      
+	delete m_pDasherView;
 	delete m_ColourIO;
 	delete m_AlphIO;
-	delete m_Colours;
+	delete m_pColours;
+	delete m_Params;
 
 	// Do NOT delete Edit box or Screen. This class did not create them.
 }
@@ -122,30 +123,63 @@ void CDasherInterface::AddColourFilename(std::string Filename)
 void CDasherInterface::CreateDasherModel()
 {
 
-  if (m_DashEditbox!=0 && m_LanguageModel!=0) {
-    delete m_DasherModel;
-    m_DasherModel = new CDasherModel(m_DashEditbox, m_LanguageModel, m_Dimensions, m_Eyetracker, m_Paused);
-    if (m_MaxBitRate>=0)
-      m_DasherModel->SetMaxBitrate(m_MaxBitRate);
-    if (ViewID!=-1)
-      ChangeView(ViewID);
-  }
+  if (m_DashEditbox!=0 && m_LanguageModelID!=-1) 
+    {
 
+      // Delete the old model and create a new one
 
+      if( m_pDasherModel != NULL ) {
+	delete m_pDasherModel;
+      }
+
+      CDasherModel::LanguageModelID NewLanguageModelID = CDasherModel::idPPM;
+
+      switch( m_LanguageModelID ) {
+      case 0:
+	NewLanguageModelID = CDasherModel::idPPM;
+	break;
+      case 1:
+	NewLanguageModelID = CDasherModel::idWord;
+	break;
+      case 2:
+	NewLanguageModelID = CDasherModel::idMixture;
+	break;	
+      }
+
+      m_pDasherModel = new CDasherModel(m_Alphabet, m_DashEditbox, NewLanguageModelID, m_Params, m_Dimensions, m_Eyetracker, m_Paused);
+      
+      // Train the new language model
+
+      string T = m_Alphabet->GetTrainingFile();
+      
+      TrainFile(m_SystemLocation+T);
+      TrainFile(m_UserLocation+T);
+
+      // Set various parameters
+      
+      m_pDasherModel->SetControlMode(m_ControlMode);
+     
+      if (m_MaxBitRate>=0)
+	m_pDasherModel->SetMaxBitrate(m_MaxBitRate);
+      if (m_ViewID!=-1)
+	ChangeView(m_ViewID);
+      
+    }
+  
 }
 
 
 void CDasherInterface::Start()
 {
     m_Paused=false;
-	if (m_DasherModel!=0) {
-		m_DasherModel->Start();
-        m_DasherModel->Set_paused(m_Paused);
+	if (m_pDasherModel!=0) {
+		m_pDasherModel->Start();
+        m_pDasherModel->Set_paused(m_Paused);
 	}
-    if (m_DasherView!=0) {
-        m_DasherView->ResetSum();
-        m_DasherView->ResetSumCounter();
-        m_DasherView->ResetYAutoOffset();
+    if (m_pDasherView!=0) {
+        m_pDasherView->ResetSum();
+        m_pDasherView->ResetSumCounter();
+        m_pDasherView->ResetYAutoOffset();
     }
 }
 
@@ -158,97 +192,105 @@ void CDasherInterface::PauseAt(int MouseX, int MouseY)
 			m_DashEditbox->CopyAll();
 	}	
 	m_Paused=true;
-    if (m_DasherModel!=0) {
-        m_DasherModel->Set_paused(m_Paused);
+    if (m_pDasherModel!=0) {
+        m_pDasherModel->Set_paused(m_Paused);
     }
 }
 
 void CDasherInterface::Halt()
 {
-  if (m_DasherModel!=0)
-    m_DasherModel->Halt();
+  if (m_pDasherModel!=0)
+    m_pDasherModel->Halt();
 }
 
 void CDasherInterface::Unpause(unsigned long Time)
 {
 	m_Paused=false;
-    if (m_DasherModel!=0) {
-		m_DasherModel->Reset_framerate(Time);
-        m_DasherModel->Set_paused(m_Paused);
+    if (m_pDasherModel!=0) {
+		m_pDasherModel->Reset_framerate(Time);
+        m_pDasherModel->Set_paused(m_Paused);
     }
-    if (m_DasherView!=0) {
-        m_DasherView->ResetSum();
-        m_DasherView->ResetSumCounter();
+    if (m_pDasherView!=0) {
+        m_pDasherView->ResetSum();
+        m_pDasherView->ResetSumCounter();
     }
 }
 
 
 void CDasherInterface::Redraw()
 {
-  if (m_DasherView!=0) {
-    m_DasherView->Render();
-    m_DasherView->Display();
+  if (m_pDasherView!=0) 
+  {
+    m_pDasherView->Render();
+    m_pDasherView->Display();
   }
 
 }
 
 
+void CDasherInterface::Redraw(int iMouseX,int iMouseY)
+{
+ 
+  if (m_pDasherView!=0) 
+  {
+    if( m_pDasherView->Render(iMouseX,iMouseY,false) ) // Only call display if something changed
+      m_pDasherView->Display();
+  }
+
+}
+
+void CDasherInterface::SetInput( CDasherInput *_pInput ) {
+
+  m_pInput = _pInput;
+
+  if (m_pDasherView!=0) 
+    m_pDasherView->SetInput( _pInput );
+}
+
+
 void CDasherInterface::TapOn(int MouseX, int MouseY, unsigned long Time)
 {
-	if (m_DasherView!=0) {
-		m_DasherView->TapOnDisplay(MouseX, MouseY, Time);
-		m_DasherView->Render();
-		if (m_DrawMouse==true) {
-		  m_DasherView->DrawMouse(MouseX, MouseY);
-		}
-		if (m_DrawMouseLine==true) {
-		  m_DasherView->DrawMouseLine(MouseX, MouseY);
-		}
-		if (m_DrawKeyboard==true) {
-		  m_DasherView->DrawKeyboard();
-		}
-		m_DasherView->Display();
+
+  //  std::cout << "Tap On" << std::endl;
+
+	if (m_pDasherView!=0) 
+	{
+		m_pDasherView->TapOnDisplay(MouseX, MouseY, Time);
+		m_pDasherView->Render(MouseX,MouseY,true);
+		m_pDasherView->Display();
 	}
-	if (m_DasherModel!=0)
-		m_DasherModel->NewFrame(Time);
+	
+	if (m_pDasherModel!=0)
+		m_pDasherModel->NewFrame(Time);
 }
 
-void CDasherInterface::DrawMousePos(int MouseX, int MouseY)
+void CDasherInterface::DrawMousePos(int iMouseX, int iMouseY, int iWhichBox )
 {
-  m_DasherView->Render();
-  if (m_DrawMouse==true) {
-    m_DasherView->DrawMouse(MouseX, MouseY);
-  }
-  if (m_DrawMouseLine==true) {
-    m_DasherView->DrawMouseLine(MouseX, MouseY);
-  }
-  if (m_DrawKeyboard==true) {
-    m_DasherView->DrawKeyboard();
-  }
-  m_DasherView->Display();
+	m_pDasherView->Render( iMouseX, iMouseY,false );
+	//if (iWhichBox!=-1)
+	//m_pDasherView->DrawMousePosBox(iWhichBox, m_iMousePosDist);
+  
+	m_pDasherView->Display();
 }
 
+
 void CDasherInterface::GoTo(int MouseX, int MouseY)
 {
-	if (m_DasherView!=0) {
-		m_DasherView->GoTo(MouseX, MouseY);
-		m_DasherView->Render();
-		if (m_DrawMouse==true) {
-		  m_DasherView->DrawMouse(MouseX, MouseY);
-		}
-		if (m_DrawKeyboard==true) {
-		  m_DasherView->DrawKeyboard();
-		}
-		m_DasherView->Display();
+	if (m_pDasherView!=0) 
+	{
+		m_pDasherView->GoTo(MouseX, MouseY);
+		m_pDasherView->Render();
+		m_pDasherView->Display();
 	}
 }
 
 void CDasherInterface::DrawGoTo(int MouseX, int MouseY)
 {
-	if (m_DasherView!=0) {
-			m_DasherView->Render();
-		m_DasherView->DrawGoTo(MouseX, MouseY);
-		m_DasherView->Display();
+	if (m_pDasherView!=0) 
+	{
+//		m_pDasherView->Render(MouseY,MouseY);
+		m_pDasherView->DrawGoTo(MouseX, MouseY);
+		m_pDasherView->Display();
 	}
 }
 
@@ -264,27 +306,29 @@ void CDasherInterface::ChangeAlphabet(const std::string& NewAlphabetID)
 	  
 	  AlphabetID = m_AlphInfo.AlphID.c_str();
 
-	  CAlphabet* old = m_Alphabet;   // So we can delete the old alphabet later
+	  std::auto_ptr<CAlphabet> ptrOld(m_Alphabet);   // So we can delete the old alphabet later
 
-	  m_Alphabet = new CCustomAlphabet(m_AlphInfo);
+	  m_Alphabet = new CAlphabet(m_AlphInfo);
 	
 	  // Apply options from alphabet
 
 	  m_TrainFile = m_UserLocation + m_Alphabet->GetTrainingFile();
 
-          if (m_ControlMode==true) {
-            m_Alphabet->AddControlSymbol();
-          }
+	  // DJW_TODO - control mode
+       //   if (m_ControlMode==true) {
+         //   m_Alphabet->AddControlSymbol();
+        //  }
 	
 	  // Recreate widgets and language model
 	  if (m_DashEditbox!=0)
 	    m_DashEditbox->SetInterface(this);
 	  if (m_DasherScreen!=0)
 	    m_DasherScreen->SetInterface(this);
-	  if (LanguageModelID!=-1 || m_LanguageModel)
-	    ChangeLanguageModel(LanguageModelID);
 	  
-	  delete old; // only delete old alphabet after telling all other objects not to use it
+	  delete m_pDasherModel;
+	  m_pDasherModel=0;
+	  CreateDasherModel();
+
 	  
 	  if (m_Alphabet->GetPalette()!= std::string("") && m_PaletteChange==true) {
 	    ChangeColours(m_Alphabet->GetPalette());
@@ -301,8 +345,8 @@ void CDasherInterface::ChangeAlphabet(const std::string& NewAlphabetID)
 	  
 	  //FIXME - this should really be done when the new view is generated
 	  //rather than fixing things up afterwards
-	  if (m_DasherView!=0) {
-	    m_DasherView->SetColourMode(m_ColourMode);
+	  if (m_pDasherView!=0) {
+	    m_pDasherView->SetColourMode(m_ColourMode);
 	  }
 	}
 
@@ -322,7 +366,11 @@ void CDasherInterface::ChangeColours(const std::string& NewColourID)
 	if (!m_ColourIO)
 		m_ColourIO = new CColourIO(m_SystemLocation, m_UserLocation, m_ColourFilenames);
 	m_ColourInfo = m_ColourIO->GetInfo(NewColourID);
-	m_Colours = new CCustomColours(m_ColourInfo);
+
+	// delete old colours on editing function
+	std::auto_ptr<CCustomColours> ptrColours(m_pColours);
+
+	m_pColours = new CCustomColours(m_ColourInfo);
 
 	ColourID=m_ColourInfo.ColourID;
 
@@ -332,7 +380,7 @@ void CDasherInterface::ChangeColours(const std::string& NewColourID)
                 m_SettingsStore->SetStringOption(Keys::COLOUR_ID, ColourID);
 
 	if (m_DasherScreen!=0) {
-	  m_DasherScreen->SetColourScheme(m_Colours);
+	  m_DasherScreen->SetColourScheme(m_pColours);
 	}
 }
 
@@ -344,46 +392,48 @@ void CDasherInterface::ChangeMaxBitRate(double NewMaxBitRate)
 {
 	m_MaxBitRate = NewMaxBitRate;
 	
-	if (m_DasherModel!=0)
-		m_DasherModel->SetMaxBitrate(m_MaxBitRate);
+	if (m_pDasherModel!=0)
+		m_pDasherModel->SetMaxBitrate(m_MaxBitRate);
 	if (m_SettingsUI!=0)
 		m_SettingsUI->ChangeMaxBitRate(m_MaxBitRate);
 	if (m_SettingsStore!=0)
 		m_SettingsStore->SetLongOption(Keys::MAX_BITRATE_TIMES100, long(m_MaxBitRate*100) );
 
-	if (m_DrawKeyboard==true && m_DasherView!=NULL) {
-	  m_DasherView->DrawKeyboard();
+	if (m_DrawKeyboard==true && m_pDasherView!=NULL) {
+	  m_pDasherView->DrawKeyboard();
 	}
 }
 
-void CDasherInterface::ChangeLanguageModel(unsigned int NewLanguageModelID)
+void CDasherInterface::ChangeLanguageModel(int NewLanguageModelID)
 {
-	LanguageModelID = NewLanguageModelID;
-	if (m_Alphabet!=0) {
-		if (m_LanguageModel)
-			m_LanguageModel->ReleaseContext(TrainContext);
-		TrainContext = 0;
-		delete m_DasherModel; // Have to delete DasherModel, or removing its LanguageModel will confuse it
-		m_DasherModel = 0;
-		delete m_LanguageModel;
-		// TODO Use LanguageModelID to decide which model to use
-		m_LanguageModel = new CPPMLanguageModel(m_Alphabet);
-		TrainContext = m_LanguageModel->GetEmptyContext();
-		string T = m_Alphabet->GetTrainingFile();
-		TrainFile(m_SystemLocation+T);
-		TrainFile(m_UserLocation+T);
-		CreateDasherModel();
-	}
+
+  if( NewLanguageModelID != m_LanguageModelID ) {
+    m_LanguageModelID = NewLanguageModelID;
+    if (m_Alphabet!=0) {
+     
+      CreateDasherModel();
+      
+      // We need to call start here so that the root is recreated, otherwise it will fail (this is probably something which needs to be fixed in a more integrated way)
+      
+      Start(); 
+      
+    }
+    if (m_SettingsUI!=0)
+      m_SettingsUI->ChangeLanguageModel(NewLanguageModelID);
+    if (m_SettingsStore!=0)
+      m_SettingsStore->SetLongOption(Keys::LANGUAGE_MODEL_ID,NewLanguageModelID);
+  }
+  
 }
 
 
 void CDasherInterface::ChangeScreen()
 {
-	if (m_DasherView!=0) {
-		m_DasherView->ChangeScreen(m_DasherScreen);
+	if (m_pDasherView!=0) {
+		m_pDasherView->ChangeScreen(m_DasherScreen);
 	} else {
-		if (ViewID!=-1)
-			ChangeView(ViewID);
+		if (m_ViewID!=-1)
+			ChangeView(m_ViewID);
 	}
 }
 
@@ -393,7 +443,7 @@ void CDasherInterface::ChangeScreen(CDasherScreen* NewScreen)
 	m_DasherScreen = NewScreen;
 	m_DasherScreen->SetFont(m_DasherFont);
 	m_DasherScreen->SetFontSize(m_DasherFontSize);
-	m_DasherScreen->SetColourScheme(m_Colours);
+	m_DasherScreen->SetColourScheme(m_pColours);
 	m_DasherScreen->SetInterface(this);
 	ChangeScreen();
 	Redraw();
@@ -402,25 +452,38 @@ void CDasherInterface::ChangeScreen(CDasherScreen* NewScreen)
 
 void CDasherInterface::ChangeView(unsigned int NewViewID)
 {
+
+  //  std::cout << "In ChangeView" << std::endl;
+
 	//TODO Use DasherViewID
-	ViewID = NewViewID;
-	if (m_DasherScreen!=0 && m_DasherModel!=0) {
-		delete m_DasherView;
+	m_ViewID = NewViewID;
+	if (m_DasherScreen!=0 && m_pDasherModel!=0) {
+
+	  //  std::cout << "Changed" << std::endl;
+
+		delete m_pDasherView;
 		if (m_Orientation==Opts::Alphabet)
-			m_DasherView = new CDasherViewSquare(m_DasherScreen, *m_DasherModel, m_LanguageModel, GetAlphabetOrientation(), m_ColourMode);
+			m_pDasherView = new CDasherViewSquare(m_DasherScreen, *m_pDasherModel, GetAlphabetOrientation(), m_ColourMode);
 		else
-			m_DasherView = new CDasherViewSquare(m_DasherScreen, *m_DasherModel, m_LanguageModel, m_Orientation, m_ColourMode);
+			m_pDasherView = new CDasherViewSquare(m_DasherScreen, *m_pDasherModel, m_Orientation, m_ColourMode);
+		m_pDasherView->SetDrawMouse(m_DrawMouse);
+		m_pDasherView->SetDrawMouseLine(m_DrawMouseLine);
+
+		if(m_pInput)
+		  m_pDasherView->SetInput(m_pInput);
+
+
 	}
 }
 
 
 void CDasherInterface::ChangeOrientation(Opts::ScreenOrientations Orientation)
 {
-	if (m_DasherView!=0) {
+	if (m_pDasherView!=0) {
 		if (Orientation==Opts::Alphabet)
-			m_DasherView->ChangeOrientation(GetAlphabetOrientation());
+			m_pDasherView->ChangeOrientation(GetAlphabetOrientation());
 		else
-			m_DasherView->ChangeOrientation(Orientation);
+			m_pDasherView->ChangeOrientation(Orientation);
 	}
 
 	if (m_Orientation != Orientation) {
@@ -511,25 +574,30 @@ void CDasherInterface::CopyAllOnStop(bool Value)
 
 void CDasherInterface::DrawMouse(bool Value)
 {
-        m_DrawMouse = Value;
+	m_DrawMouse = Value;
+	if (m_pDasherView)
+		m_pDasherView->SetDrawMouse(Value);
+
 	if (m_SettingsUI!=0)
-                m_SettingsUI->DrawMouse(Value);
+		m_SettingsUI->DrawMouse(Value);
 	if (m_SettingsStore!=0)
-	        m_SettingsStore->SetBoolOption(Keys::DRAW_MOUSE, Value);
+		m_SettingsStore->SetBoolOption(Keys::DRAW_MOUSE, Value);
 }
 
 void CDasherInterface::DrawMouseLine(bool Value)
 {
-        m_DrawMouseLine = Value;
+    m_DrawMouseLine = Value;
+	if (m_pDasherView)
+		m_pDasherView->SetDrawMouseLine(Value);
 	if (m_SettingsUI!=0)
-                m_SettingsUI->DrawMouseLine(Value);
+        m_SettingsUI->DrawMouseLine(Value);
 	if (m_SettingsStore!=0)
-	        m_SettingsStore->SetBoolOption(Keys::DRAW_MOUSELINE, Value);
+	    m_SettingsStore->SetBoolOption(Keys::DRAW_MOUSELINE, Value);
 }
 
 void CDasherInterface::StartOnSpace(bool Value)
 {
-        m_StartSpace = Value;
+    m_StartSpace = Value;
 	if (m_SettingsUI!=0)
 	  m_SettingsUI->StartOnSpace(Value);
 	if (m_SettingsStore!=0)
@@ -552,8 +620,8 @@ void CDasherInterface::KeyControl(bool Value)
 	  m_SettingsUI->KeyControl(Value);
 	if (m_SettingsStore!=0)
 	  m_SettingsStore->SetBoolOption(Keys::KEY_CONTROL, Value);
-	if (m_DasherView!=0)
-	  m_DasherView->SetKeyControl(Value);
+	if (m_pDasherView!=0)
+	  m_pDasherView->SetKeyControl(Value);
 }
 
 void CDasherInterface::WindowPause(bool Value)
@@ -566,19 +634,25 @@ void CDasherInterface::WindowPause(bool Value)
 
 void CDasherInterface::ControlMode(bool Value)
 {
-  m_ControlMode=Value;
-  if (m_SettingsStore!=0)
-    m_SettingsStore->SetBoolOption(Keys::CONTROL_MODE, Value);
-  if (m_Alphabet!=0) {
-    if (Value==true) {
-      m_Alphabet->AddControlSymbol();
-    } else {
-      m_Alphabet->DelControlSymbol();
-    }
-  }
+	m_ControlMode=Value;
+	if (m_SettingsStore!=0)
+		m_SettingsStore->SetBoolOption(Keys::CONTROL_MODE, Value);
+	if (m_pDasherModel!=0) 
+	{
+		m_pDasherModel->SetControlMode(Value);
+		// DJW_TODO - control symbol
+//		if (Value==true) 
+//		{
+//			m_Alphabet->AddControlSymbol();
+//		} 
+//		else 
+//		{
+//			m_Alphabet->DelControlSymbol();
+//		}
+	}
 
-  if (m_SettingsUI!=0)
-    m_SettingsUI->ControlMode(Value);
+	if (m_SettingsUI!=0)
+		m_SettingsUI->ControlMode(Value);
 
   Start();
   Redraw();
@@ -593,13 +667,25 @@ void CDasherInterface::KeyboardMode(bool Value)
     m_SettingsStore->SetBoolOption(Keys::KEYBOARD_MODE, Value);
 }
 
+void CDasherInterface::SetDrawMousePosBox(int iWhich)
+{
+	m_iMousePosBox = iWhich;
+	if (m_pDasherView)
+		m_pDasherView->SetDrawMousePosBox(iWhich);
+	
+}
+
 void CDasherInterface::MouseposStart(bool Value)
 {
   m_MouseposStart=Value;
+  if (!Value && m_pDasherView)
+	  m_pDasherView->SetDrawMousePosBox(0);
+  
   if (m_SettingsUI!=0)
     m_SettingsUI->MouseposStart(Value);
   if (m_SettingsStore!=0)
     m_SettingsStore->SetBoolOption(Keys::MOUSEPOS_START, Value);
+
 }
 
 void CDasherInterface::OutlineBoxes(bool Value)
@@ -659,8 +745,8 @@ void CDasherInterface::SetEditFont(string Name, long Size)
 
 void CDasherInterface::SetUniform(int Value)
 {
-  if( m_LanguageModel != NULL )
-    m_LanguageModel->SetUniform(Value);
+  if( m_pDasherModel != NULL )
+    m_pDasherModel->SetUniform(Value);
   if (m_SettingsStore!=0) {
     m_SettingsStore->SetLongOption(Keys::UNIFORM, Value);
   }
@@ -675,9 +761,11 @@ void CDasherInterface::SetYScale(int Value)
 
 void CDasherInterface::SetMousePosDist(int Value)
 {
+	m_iMousePosDist = Value;
 	if (m_SettingsStore!=0) {
 		m_SettingsStore->SetLongOption(Keys::MOUSEPOSDIST, Value);
 	}
+
 }
 
 void CDasherInterface::SetDasherFont(string Name)
@@ -709,8 +797,8 @@ void CDasherInterface::SetDasherDimensions(bool Value)
         m_Dimensions=Value;
 	if (m_SettingsStore!=0)
 		m_SettingsStore->SetBoolOption(Keys::DASHER_DIMENSIONS, Value);
-	if (m_DasherModel!=0) {
-	         m_DasherModel->Set_dimensions(Value);
+	if (m_pDasherModel!=0) {
+	         m_pDasherModel->Set_dimensions(Value);
 	}
 	if (m_SettingsUI!=0) {
 	         m_SettingsUI->SetDasherDimensions(Value);
@@ -722,14 +810,28 @@ void CDasherInterface::SetDasherEyetracker(bool Value)
         m_Eyetracker=Value;
 	if (m_SettingsStore!=0)
 		m_SettingsStore->SetBoolOption(Keys::DASHER_EYETRACKER, Value);
-	if (m_DasherModel!=0) {
-	         m_DasherModel->Set_eyetracker(Value);
+	if (m_pDasherModel!=0) {
+	         m_pDasherModel->Set_eyetracker(Value);
 	}
 	if (m_SettingsUI!=0) {
 	         m_SettingsUI->SetDasherEyetracker(Value);
 	}	  
 }
 
+void CDasherInterface::SetTruncation( int Value ) {
+
+  //  std::cout << "In SetTruncation: " << m_pDasherView << std::endl;
+
+  if( m_pDasherView ) {
+    m_pDasherView->SetTruncation( Value );
+  }
+}
+
+void CDasherInterface::SetTruncationType( int Value ) {
+  if( m_pDasherView ) {
+    m_pDasherView->SetTruncationType( Value );
+  }
+}
 
 unsigned int CDasherInterface::GetNumberSymbols()
 {
@@ -848,8 +950,8 @@ void CDasherInterface::ChangeEdit(CDashEditbox* NewEdit)
 
 void CDasherInterface::ColourMode(bool Value)
 {
-  if (m_DasherView!=0) {
-    m_DasherView->SetColourMode(Value);
+  if (m_pDasherView!=0) {
+    m_pDasherView->SetColourMode(Value);
   }
   m_ColourMode=Value;
   Start();
@@ -858,7 +960,7 @@ void CDasherInterface::ColourMode(bool Value)
 
 void CDasherInterface::Train(string* TrainString, bool IsMore)
 {
-	m_LanguageModel->LearnText(TrainContext, TrainString, IsMore);
+//	m_pDasherModel->LearnText(TrainContext, TrainString, IsMore);
 	return;
 }
 
@@ -871,6 +973,7 @@ void CDasherInterface::Train(string* TrainString, bool IsMore)
 */
 void CDasherInterface::TrainFile(string Filename)
 {
+
 	if (Filename=="")
 		return;
 	
@@ -883,18 +986,29 @@ void CDasherInterface::TrainFile(string Filename)
 	string StringBuffer;
 	int NumberRead;
 	
+	vector<symbol> Symbols;
+	
+	CDasherModel::CTrainer* pTrainer = m_pDasherModel->GetTrainer();
 	do {
 		NumberRead = fread(InputBuffer, 1, BufferSize-1, InputFile);
 		InputBuffer[NumberRead] = '\0';
 		StringBuffer += InputBuffer;
-		if (NumberRead == (BufferSize-1)) {
-		  m_LanguageModel->LearnText(TrainContext, &StringBuffer, true);
-		} else {
-		  m_LanguageModel->LearnText(TrainContext, &StringBuffer, false);
-		}
+		bool bIsMore = false;
+		if (NumberRead == (BufferSize-1)) 
+			bIsMore = true;
+
+		Symbols.clear();
+		m_Alphabet->GetSymbols(&Symbols, &StringBuffer, bIsMore);
+		
+		pTrainer->Train( Symbols );
+		
+		
 	} while (NumberRead==BufferSize-1);
 
+	delete pTrainer;
+
 	fclose(InputFile);
+
 }
 
 void CDasherInterface::GetFontSizes(std::vector<int> *FontSizes) const
@@ -926,33 +1040,63 @@ double CDasherInterface::GetCurFPS()
 
 void CDasherInterface::AddControlTree(ControlTree *controltree)
 {
-  m_DasherModel->NewControlTree(controltree);
+  m_pDasherModel->NewControlTree(controltree);
 }
 
 void CDasherInterface::Render()
 {
-  if (m_DasherView!=0)
-    m_DasherView->Render();
+//  if (m_pDasherView!=0)
+ //   m_pDasherView->Render();
 }
 
 int CDasherInterface::GetOneButton() {
-  if (m_DasherView!=0) {
-    return m_DasherView->GetOneButton();
+  if (m_pDasherView!=0) {
+    return m_pDasherView->GetOneButton();
   }
   return -1;
 }
 
 int CDasherInterface::GetAutoOffset() {
-  if (m_DasherView!=0) {
-    return m_DasherView->GetAutoOffset();
+  if (m_pDasherView!=0) {
+    return m_pDasherView->GetAutoOffset();
   }
   return -1;
 }
 
 
 void CDasherInterface::SetOneButton(int Value) {
-  if (m_DasherView!=0) {
-    m_DasherView->SetOneButton(Value);
+  if (m_pDasherView!=0) {
+    m_pDasherView->SetOneButton(Value);
   }
 }
 
+/////////////////////////////////////////////////////////////////////////////
+
+double CDasherInterface::GetNats() const
+{
+	if( m_pDasherModel )
+		return m_pDasherModel->GetNats();
+	else
+		return 0.0;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+
+void CDasherInterface::ResetNats() 
+{
+	if( m_pDasherModel )
+		m_pDasherModel->ResetNats();
+}
+
+/////////////////////////////////////////////////////////////////////////////
+
+void CDasherInterface::ChangeLMOption( const std::string &pname, long int Value ) 
+{
+  m_Params->SetValue( pname, Value );
+
+  if (m_SettingsStore!=0)
+    m_SettingsStore->SetLongOption( pname, Value);
+
+  if (m_SettingsUI!=0)
+     m_SettingsUI->ChangeLMOption( pname, Value );
+}
