@@ -9,6 +9,8 @@
 #ifndef __FrameRate_h__
 #define __FrameRate_h__
 
+#include "../Common/Common.h"
+
 // keeps track of framerate
 // computes the Steps parameter
 // computes RXmax - which controls the maximum rate of zooming in
@@ -21,6 +23,15 @@ class CFrameRate {
 		~CFrameRate() {};
 		void Initialise();
 		double Rxmax() const {return m_dRXmax;}
+
+		/// Get the minimum size of the target viewport
+		///
+		/// TODO: Eventually fix this so that it uses integer maths internally.
+
+		myint MinSize( myint t ) const {
+		  return myint(t / m_dRXmax);
+		};
+
 		int Steps() const {return m_iSteps;}
 		double Framerate() const {return m_dFr;}
 		void Reset(unsigned long Time);
@@ -98,11 +109,10 @@ inline void CFrameRate::NewFrame(unsigned long Time)
 		if (m_iSteps==0)
 			m_iSteps=1;
 
-	//	dchar debug[256];
-	//		_stprintf(debug,TEXT("fr %f Steps %d samples %d time2 %d rxmax %f\n"),fr,Steps,samples,time2,RXmax);
-	//	OutputDebugString(debug);
+		DASHER_TRACEOUTPUT("Fr %f Steps %d Samples %d Time2 %d rxmax %f\n",m_dFr,m_iSteps,m_iSamples,m_iTime2,m_dRXmax);
 	
 	}
+
 }
 
 inline void CFrameRate::Reset(unsigned long Time)
