@@ -3,6 +3,8 @@
 #include "DashEdit.h"
 #include "DasherTypes.h"
 
+#include <iostream>
+
 #include "libdasher.h"
 #include "libdasher_private.h"
 
@@ -29,9 +31,11 @@ void (*colour_scheme_callback)(int, int*, int*, int*) = NULL;
 void (*draw_rectangle_callback)(int, int, int, int, int, Opts::ColorSchemes) = NULL;
 void (*draw_polyline_callback)(Dasher::CDasherScreen::point*, int) = NULL;
 void (*draw_colour_polyline_callback)(Dasher::CDasherScreen::point*, int, int) = NULL;
+void (*draw_colour_polygon_callback)(Dasher::CDasherScreen::point*, int, int) = NULL;
 void (*draw_text_callback)(symbol, int, int, int) = NULL;
 void (*draw_text_string_callback)(std::string, int, int, int) = NULL;
-void (*text_size_callback)(symbol, int*, int*, int) = NULL;
+void (*text_size_callback)(const std::string &String, int*, int*, int) = NULL;
+void (*send_marker_callback)( int iMarker ) = NULL;
 
 void (*edit_output_callback)(symbol) = NULL;
 void (*edit_outputcontrol_callback)(void*, int) = NULL;
@@ -86,7 +90,7 @@ void handle_display()
     display_callback();
 }
 
-void handle_colour_scheme(CCustomColours *Colours)
+void handle_colour_scheme(const CCustomColours *Colours)
 {
   if (colour_scheme_callback==NULL || Colours==NULL)
     return;
@@ -101,9 +105,9 @@ void handle_colour_scheme(CCustomColours *Colours)
     blues[i]=Colours->GetBlue(i);
   }
   colour_scheme_callback(numcolours,reds,greens,blues);
-  delete reds;
-  delete greens;
-  delete blues;
+  delete[] reds;
+  delete[] greens;
+  delete[] blues;
 }
 
 void handle_draw_rectangle(int x1, int y1, int x2, int y2, int Color, Opts::ColorSchemes ColorScheme)
@@ -125,22 +129,35 @@ void handle_draw_colour_polyline(Dasher::CDasherScreen::point* Points, int Numbe
     draw_colour_polyline_callback( Points, Number, Colour );
 }
 
+void handle_draw_colour_polygon(Dasher::CDasherScreen::point* Points, int Number, int Colour)
+{
+  if( draw_colour_polygon_callback != NULL )
+    draw_colour_polygon_callback( Points, Number, Colour );
+}
+
 void handle_draw_text(symbol Character, int x1, int y1, int size)
 {
   if( draw_text_callback != NULL )
     draw_text_callback( Character, x1, y1, size );
 }
 
-void handle_draw_text(std::string String, int x1, int y1, int size)
+void handle_draw_text(const std::string &String, int x1, int y1, int size)
 {
   if( draw_text_string_callback != NULL )
     draw_text_string_callback( String, x1, y1, size );
 }
 
-void handle_text_size(symbol Character, int* Width, int* Height, int Size)
+void handle_send_marker( int iMarker ) {
+  if( send_marker_callback != NULL )
+    send_marker_callback( iMarker );
+}
+
+void handle_text_size(const std::string &String, int* Width, int* Height, int Size)
 {
   if( text_size_callback != NULL )
-    text_size_callback(Character, Width, Height, Size);
+    text_size_callback(String, Width, Height, Size);
+  else
+    *Width = *Height = 0;
 }
 
 void handle_edit_output(symbol Character)
@@ -234,12 +251,12 @@ void dasher_late_initialise( int _width, int _height)
   dsc = new dasher_screen( _width, _height );
   interface->ChangeScreen( dsc );
 
-  ded = new dasher_edit;
-  interface->ChangeEdit( ded );
-
   dss = new dasher_settings_store;
   interface->SetSettingsStore( dss );
 
+  ded = new dasher_edit;
+  interface->ChangeEdit( ded );
+
   dasher_start();
   dasher_redraw();
 
@@ -271,6 +288,7 @@ void dasher_set_parameter_string( string_param p, const char *value )
       interface->SetSystemLocation( s );
       break;
     case STRING_ALPHABET:
+      std::cout << "Changing alphabet to " << s << std::endl;
       interface->ChangeAlphabet( s );
       break;
     case STRING_COLOUR:
@@ -332,6 +350,30 @@ void dasher_set_parameter_int( int_param p, long int value )
     case INT_UNIFORM:
       interface->SetUniform(value);
       break;
+    case INT_LM_MAXORDER:
+      interface->ChangeLMOption( "LMMaxOrder", value ); // FIXME - repeated translation of references to parameters is silly
+      break;
+    case INT_LM_ALPHA:
+      interface->ChangeLMOption( "LMAlpha", value );
+      break;
+    case INT_LM_BETA:
+      interface->ChangeLMOption( "LMBeta", value );
+      break;
+    case INT_LM_EXCLUSION:
+      interface->ChangeLMOption( "LMExclusion", value );
+      break;
+    case INT_LM_UPDATE_EXCLUSION:
+      interface->ChangeLMOption( "LMUpdateExclusion", value );
+      break;
+    case INT_LM_MIXTURE:
+      interface->ChangeLMOption( "LMMixture", value );
+      break;
+    case INT_TRUNCATION:
+      interface->SetTruncation( value );
+      break;
+    case INT_TRUNCATIONTYPE:
+      interface->SetTruncationType( value );
+      break;
     default:
       break;
     }
@@ -420,6 +462,10 @@ void dasher_set_encoding( Dasher::Opts::FileEncodingFormats encoding )
   interface->SetFileEncoding( encoding );
 }
 
+//void dasher_set_lm_parameter( const char *pname, int value ) {
+//  interface->ChangeLMOption( pname, value );
+//}
+
 void dasher_train_file( const char *filename )
 {
   interface->TrainFile(filename);
@@ -526,6 +572,11 @@ void dasher_set_draw_colour_polyline_callback(void (*_cb)(Dasher::CDasherScreen:
   draw_colour_polyline_callback = _cb;
 }
 
+void dasher_set_draw_colour_polygon_callback(void (*_cb)(Dasher::CDasherScreen::point*, int, int) )
+{
+  draw_colour_polygon_callback = _cb;
+}
+
 void dasher_set_draw_text_callback(void (*_cb)(symbol, int, int, int))
 {
   draw_text_callback = _cb;
@@ -536,7 +587,12 @@ void dasher_set_draw_text_string_callback(void (*_cb)(std::string, int, int, int
   draw_text_string_callback = _cb;
 }
 
-void dasher_set_text_size_callback(void (*_cb)(symbol, int*, int*, int))
+void dasher_set_send_marker_callback( void (*_cb)( int ) )
+{
+  send_marker_callback = _cb;
+}
+
+void dasher_set_text_size_callback(void (*_cb)(const std::string &, int*, int*, int))
 {
   text_size_callback = _cb;
 }
@@ -617,9 +673,9 @@ void dasher_tap_on( int x, int y, unsigned long int time )
   interface->TapOn( x, y, time );
 }
 
-void dasher_draw_mouse_position( int x, int y)
+void dasher_draw_mouse_position( int x, int y, int iWhich)
 {
-  interface->DrawMousePos( x, y );
+  interface->DrawMousePos( x, y, iWhich  );
 }
 
 void dasher_go_to( int x, int y )
@@ -647,6 +703,16 @@ void dasher_pause( int x, int y )
   interface->PauseAt( x, y );
 }
 
+void dasher_reset_nats()
+{
+  interface->ResetNats();
+}
+
+double dasher_get_nats()
+{
+  return interface->GetNats();
+}
+
 // FIXME - this should not use c++ style strings
 
 string dasher_get_display_text( symbol Character )
@@ -720,3 +786,8 @@ void add_colour_filename(const char* filename)
 {
   interface->AddColourFilename(filename);
 }
+
+void dasher_set_input( CDasherInput *_pInput ) {
+
+  interface->SetInput( _pInput );
+}
