@@ -12,18 +12,20 @@ void handle_parameter_bool( bool_param p, bool value );
 
 void handle_blank();
 void handle_display();
-void handle_colour_scheme(CCustomColours *Colours);
+void handle_colour_scheme(const CCustomColours *Colours);
 
 void handle_draw_rectangle(int x1, int y1, int x2, int y2, int Color, Opts::ColorSchemes ColorScheme);
 void handle_draw_polyline(Dasher::CDasherScreen::point* Points, int Number);
 void handle_draw_colour_polyline(Dasher::CDasherScreen::point* Points, int Number, int Colour);
+void handle_draw_colour_polygon(Dasher::CDasherScreen::point* Points, int Number, int Colour);
 void handle_draw_text(symbol Character, int x1, int y1, int size);
-void handle_draw_text(std::string String, int x1, int y1, int size);
-void handle_text_size(symbol Character, int* Width, int* Height, int Size);
+void handle_draw_text(const std::string &String, int x1, int y1, int size);
+void handle_text_size(const std::string &String, int* Width, int* Height, int Size);
 void handle_edit_output(symbol Character);
 void handle_edit_outputcontrol(void* pointer, int data);
 void handle_edit_delete(symbol Character);
 void handle_get_new_context( std::string &str, int max );
+void handle_send_marker( int iMarker );
 
 void handle_clipboard( clipboard_action act );
 
@@ -60,6 +62,32 @@ class dasher_ui : public CDasherSettingsInterface
       handle_parameter_int( INT_LANGUAGEMODEL, NewLanguageModelID );
     };
 
+  void ChangeLMOption( const std::string &pname, long int Value )
+    {
+      // FIXME - all this should be much more flexible
+
+      if( pname == "LMMaxOrder" ) {
+	handle_parameter_int( INT_LM_MAXORDER, Value ); 
+      }
+      else if( pname == "LMAlpha" ) {
+	handle_parameter_int( INT_LM_ALPHA, Value ); 
+      }
+      else if( pname == "LMBeta" ) {
+	handle_parameter_int( INT_LM_BETA, Value ); 
+      }
+      else if( pname == "LMExclusion" ) {
+	handle_parameter_int( INT_LM_EXCLUSION, Value ); 
+      }
+      else if( pname == "LMUpdateExclusion" ) {
+	handle_parameter_int( INT_LM_UPDATE_EXCLUSION, Value ); 
+      }
+      else if( pname == "LMMixture" ) {
+	handle_parameter_int( INT_LM_MIXTURE, Value ); 
+      }
+
+
+    };
+
   void ChangeView(unsigned int NewViewID)
     {
       handle_parameter_int( INT_VIEW, NewViewID );
@@ -206,6 +234,12 @@ class dasher_ui : public CDasherSettingsInterface
     {
       handle_parameter_bool( BOOL_PALETTECHANGE, Value );
     }
+  void SetTruncation( int Value ) {
+    handle_parameter_int( INT_TRUNCATION, Value );
+  }
+  void SetTruncationType( int Value ) {
+    handle_parameter_int( INT_TRUNCATION, Value );
+  }
 };
 
 
@@ -213,7 +247,7 @@ class dasher_screen : public CDasherScreen
 {
  public:
 
-  dasher_screen( int _width, int _height )
+  dasher_screen( screenint _width, screenint _height )
     : CDasherScreen( _width, _height )
     {
     };
@@ -226,7 +260,7 @@ class dasher_screen : public CDasherScreen
     {
     };
 	
-  Dasher::Opts::FontSize GetFontSize()
+  Dasher::Opts::FontSize GetFontSize() const
     {
       if (dasherfontsize==0)
 	return Dasher::Opts::FontSize(1);
@@ -234,25 +268,31 @@ class dasher_screen : public CDasherScreen
 	return Dasher::Opts::FontSize(dasherfontsize);
     };
 
-  void TextSize(symbol Character, int* Width, int* Height, int Size) const
+  void TextSize(const std::string &String, screenint* Width, screenint* Height, int Size) const
     {
-      handle_text_size( Character, Width, Height, Size );
+      handle_text_size( String, Width, Height, Size );
     };
 
-  void DrawText(symbol Character, int x1, int y1, int Size) const
+  void DrawString(symbol Character, screenint x1, screenint y1, int Size) const
     {
+      // It's possible that this is no longer used...
+
       handle_draw_text( Character, x1, y1, Size );
     };
 
-  void DrawText(std::string String, int x1, int y1, int Size) const
+  void DrawString(const std::string &String, screenint x1, screenint y1, int Size) const
     {
       handle_draw_text( String, x1, y1, Size );
     };
 
-  void DrawRectangle(int x1, int y1, int x2, int y2, int Color, Opts::ColorSchemes ColorScheme) const
+  void DrawRectangle(screenint x1, screenint y1, screenint x2, screenint y2, int Color, Opts::ColorSchemes ColorScheme) const
     {
       handle_draw_rectangle( x1, y1, x2, y2, Color, ColorScheme);
     };
+
+  void SendMarker( int iMarker ) {
+    handle_send_marker( iMarker );
+  };
 	
   void Polyline(point* Points, int Number) const
     {
@@ -263,7 +303,12 @@ class dasher_screen : public CDasherScreen
     {
       handle_draw_colour_polyline( Points, Number, Colour );
     };
-	
+
+  void Polygon(point* Points, int Number, int Colour) const
+    {
+      handle_draw_colour_polygon( Points, Number, Colour );
+    };
+
   void DrawPolygon(point* Points, int Number, int Color, Opts::ColorSchemes ColorScheme) const
     {
     };
@@ -278,7 +323,7 @@ class dasher_screen : public CDasherScreen
       handle_display();
     };
 
-  void SetColourScheme(CCustomColours *Colours)
+  void SetColourScheme(const CCustomColours *Colours)
     {
       handle_colour_scheme(Colours);
     };
