@@ -219,7 +219,7 @@ void update_colours()
   }
 
   colourscheme=dasher_get_current_colours();
-  const int colourlist_size=128;
+  const int colourlist_size=1024;
   const char *colourlist[ colourlist_size ];
   int colour_count = dasher_get_colours( colourlist, colourlist_size );
   for (int i=0; i<colour_count; i++) {
@@ -337,9 +337,9 @@ generate_preferences(GtkWidget *widget, gpointer user_data) {
 
   int alphabet_count, colour_count;
 
-  const int alphabetlist_size = 128;
+  const int alphabetlist_size = 1024;
   const char *alphabetlist[ alphabetlist_size ];
-  const int colourlist_size=128;
+  const int colourlist_size=1024;
   const char *colourlist[ colourlist_size ];
   GtkTreeIter alphiter, colouriter;
 
@@ -1770,11 +1770,35 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data)
   gchar *authors[] = {
     "Chris Ball",
     "Phil Cowans",
+    "Frederik Eaton",
+    "Behdad Esfahbod",
     "Matthew Garrett",
+    "Chris Hack",
     "David MacKay",
     "Iain Murray",
+    "Takashi Kaburagi",
+    "Keith Vertanen",
     "Hanna Wallach",
     "David Ward",
+    "Brian Williams",
+    "Yasser H. Abdel-Haleem (Arabic training text)",
+    "Peter Baert (Dutch and French training texts)",
+    "Johan Bouvin (Swedish training text)",
+    "Gilles Brodeur (French training text)",
+    "Behdad Esfahbod (Persian training text)",
+    "Gilles-Maurice De Schryver (Training texts for various South African languages)",
+    "Edmund Grimley Evans (Esperanto training text)",
+    "Matteo Fasanella (Italian training text)",
+    "Leena Krohn (Finnish training text)",
+    "Michael Muhlhaus (German training text)",
+    "Pantelis Nasikas (Greek training text)",
+    "Blazej L. Popielski (Polish training text)",
+    "Andrea Prado (Spanish training text)",
+    "Nikon Rasumov (Russian Latin training text)",
+    "Alban Rrustemi (Albanian training text)",
+    "Martin Whittle (Brazilian Portugese training text)",
+    "Ole Winther (Danish training text)",
+    "Alex Ziranov (Russian training text)",
     NULL
   };
     
@@ -1783,6 +1807,18 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data)
     "Chris Ball",
     "Matthew Garrett",
     "David MacKay",
+    "Hugh M. Allen (Norwegian website translation)",
+    "Claudia V. Bourda (Spanish website translation)",
+    "Radoslaw Cichocki (Polish website translation)",
+    "Fabio Fonseca (Portugese website translation)",
+    "Dan Witzner Hansen (Danish website translation)",
+    "Andrew B. Kuliapin (Russian website translation)",
+    "Xiaoying Liang (Chinese website translation)",
+    "Simone Mangano (Italian website translation)",
+    "Christoforos Moutafis (Greek website translation)",
+    "Julius Roelcke (German website translation)",
+    "Louis Tan (Dutch website translation)",
+    "Marc Xhrouet (French website translation)",
     NULL
   };
 
